/*---------------------------------------------------------------------------\
| Project:	Generic LV2 PIC18F458 CAN node firmware
|
| File:		can.h
|
| Descr.:	Includes for all CAN routines
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _CAN_H          // Don't include more than once (last line = #endif)
#define _CAN_H


/*********************************************************************
 * #defines
 ********************************************************************/

// CAN Message ID's max and min values for this CAN node. Application
// dependent - defaults = LV2 CAN message bounds.

#define MIN_MESSAGE_ID	0b0000000100000000 // 0x080
#define	MAX_MESSAGE_ID	0b1111111111100000 // 0x7FF

// Priority cutoffs for sorting messages into the 3 transmit queues;
// see SendCanMessage(). High priority messages have ID's lower than
// or equal to the first cutoff, normal priority messages have
// ID's lower than or equal to the normal priority cutoffs but less
// than the high priority cutoffs. Anything less than the normal
// priority cutoff is considered "low priority".

#define	HIGH_PRIORITY_TX_CUTOFF		0b0000001010101010	// 1/3 =  682
#define	NORMAL_PRIORITY_TX_CUTOFF	0b0000010101010100	// 2/3 = 1364

// Size of the CAN queues.

#define CAN_TX2_QUEUE_SIZE	5
#define CAN_TX1_QUEUE_SIZE	5
#define CAN_TX0_QUEUE_SIZE	5

#define	CAN_RX_QUEUE_SIZE	5

/*********************************************************************
 * Typedefs and Enumerations
 ********************************************************************/

/*---------------------------------------------------------------------------\
| "CanHeader_t" - CAN Header type
|
| This union provides a data type for the CAN message ID, DLC (Data Length
| Code) and RTR (Request Trasmit) bits. During initialization of the CAN
| protocol unit, we'll use it exclusively for the Message ID aspect - but
| when we talk about queues of CAN messages, we'll use the all aspects of
| the header. Examples:
|
|	CanHeader_t	foo;
|	foo.header	=	all 16bits of the header
|	foo.bytes.high	=	low byte of the Message ID
\---------------------------------------------------------------------------*/


typedef union
{
    uint16_t	all;

	// WARNING: "low" versus "high may be compiler specific. For the case of
	// MCC18 v2.00, "low" is indeed the lowest byte in the address space and
	// "high" is the next.

    struct
    {
        uint8_t low;			// Bits 0 -  8 of the 16bit header
        uint8_t high;			// Bits 8 - 15 of the 16bit header
    } bytes;

    struct						//  ASSUMING LITTLE ENDIAN!!!!!
    {
        uint8_t	length:4;		// Bits 0 -  3: Number of data bytes in message
        uint8_t	request:1;		// Bits 4 -  4: Remote Frame Request bit
        uint8_t	idlow:3;		// Bits 5 - 7: CAN ID, Bit 5 = LSB, Bit15 = MSB --- IS THIS TRUE??!?!?! COMPILER DEPENDENT!!!
        uint8_t	idhigh;			// Bits 8 - 15: CAN ID, Bit 5 = LSB, Bit15 = MSB --- IS THIS TRUE??!?!?! COMPILER DEPENDENT!!!
    } bits;

} CanHeader_t;


/*---------------------------------------------------------------------------\
| "CanMessage_t" - CAN Message type
|
| We're providing a data type of a CanHeader_t with an array of 8 bytes.
| Inefficient you say? You bet, we say, but it's faster, less prone to horrible
| errors AND heck we've got 1.5K to burn, baby, burn! Compared to the 12C508
| with 128 registers, we might as well add a web server to go with it!
\---------------------------------------------------------------------------*/

typedef	struct
{
	CanHeader_t	header;
	uint8_t		data[8];
} CanMessage_t;



/*---------------------------------------------------------------------------\
| "CAN_MODES" - CAN Protocol Unit "modes" enum (See CANCON, CANSTAT registers)
|
| These are all the different modes we can put the CAN protocol Unit into
| using the CANCON register. We can use CANSTAT to read the mode of the unit
| as well (which is necessary, since the transition to a new state is
| determined by the state machine and not our command in CANCON.
\---------------------------------------------------------------------------*/


enum CAN_MODES
{
    CAN_MODE_BITS    = 0b11100000,  // Mask to access mode bits
    CAN_NORMAL_MODE  = 0b00000000,
    CAN_SLEEP_MODE   = 0b00100000,
    CAN_LOOP_MODE    = 0b01000000,
    CAN_LISTEN_MODE  = 0b01100000,
    CAN_CONFIG_MODE  = 0b10000000
};


/*---------------------------------------------------------------------------\
| "CAN_BITRATE_TYPE" - CAN bus bitrate ("baud") enums.
|
| You may think we're lame for only having 1Mbps and 125kbps, put would you
| use the CAN bus at a lower bitrate than 1Mbps? We sure as heck wouldn't. But
| just in case the 125kbps is there. The register settings that this enum
| calls for can be found in the InitializeCan routine.
\---------------------------------------------------------------------------*/

enum CAN_BITRATE_TYPE
{
	CAN_BITRATE_125KBPS,
	CAN_BITRATE_1MBPS
};


/*********************************************************************
 *
 * enum CAN_RX_MSG_FLAGS
 *
 * This enumeration values define flags related to reception of a CAN
 * message.  There could be more than one this flag
 * ANDed together to form multiple flags.
 * If a particular bit is set, corresponding meaning is TRUE or else
 * it will be FALSE.
 *
 * e.g.
 *      if (MsgFlag & CAN_RX_OVERFLOW)
 *      {
 *          // Receiver overflow has occured.  We have lost previous
 *          // message.
 *          ...
 *      }
 *
 ********************************************************************/

enum CAN_RX_MSG_FLAGS
{
    CAN_RX_FILTER_BITS  = 0b00000111,   // Use this to access filter
                                        // bits
    CAN_RX_FILTER_1     = 0b00000000,
    CAN_RX_FILTER_2     = 0b00000001,
    CAN_RX_FILTER_3     = 0b00000010,
    CAN_RX_FILTER_4     = 0b00000011,
    CAN_RX_FILTER_5     = 0b00000100,
    CAN_RX_FILTER_6     = 0b00000101,

    CAN_RX_OVERFLOW     = 0b00001000,   // Set if Overflowed else
                                        // cleared

    CAN_RX_INVALID_MSG  = 0b00010000,   // Set if invalid else
                                        // cleared

    CAN_RX_XTD_FRAME    = 0b00100000,   // Set if XTD message else
                                        // cleared

    CAN_RX_RTR_FRAME    = 0b01000000,   // Set if RTR message else
                                        // cleared

    CAN_RX_DBL_BUFFERED = 0b10000000    // Set if this message was
                                        // hardware double-buffered
};


/*********************************************************************
 *
 * enum CAN_MASK
 *
 * This enumeration values define mask codes.  Routine CANSetMask()
 * requires this code as one of its arguments.  These enumerations
 * must be used by itself i.e. it cannot be ANDed to form multiple
 * values.
 *
 * ADG 5/28/02 - Added RX_, set to B0 and B1
 ********************************************************************/

enum CAN_MASK
{
     CAN_RX_MASK_B0,
     CAN_RX_MASK_B1
};


/*********************************************************************
 *
 * enum CAN_FILTER
 *
 * This enumeration values define filter codes.  Routine CANSetFilter
 * requires this code as one of its arguments.  These enumerations
 * must be used by itself
 * i.e. it cannot be ANDed to form multiple values.
 *
 * ADG 5/28/02 - Added RX_, set to B0 and B1, F0 - 5 to be explicit
 ********************************************************************/

enum CAN_FILTER
{
    CAN_RX_FILTER_B0_F0,
    CAN_RX_FILTER_B0_F1,
    CAN_RX_FILTER_B1_F2,
    CAN_RX_FILTER_B1_F3,
    CAN_RX_FILTER_B1_F4,
    CAN_RX_FILTER_B1_F5
};



/*********************************************************************
 * Function Prototypes
 ********************************************************************/


// Initialize the CAN peripheral with the requested bitrate
// and setup the masks and filters on the receive buffers.

 void InitializeCan(enum CAN_BITRATE_TYPE canBitRate,
                    CanHeader_t canMaskB0,
                    CanHeader_t canMaskB1,
                    CanHeader_t CanFilterB0F0,
                    CanHeader_t CanFilterB0F1,
                    CanHeader_t CanFilterB1F2,
                    CanHeader_t CanFilterB1F3,
                    CanHeader_t CanFilterB1F4,
                    CanHeader_t CanFilterB1F5 );

// Do the nitty gritty time details to set up bit rates and bit timing

void SetCanTiming (enum CAN_BITRATE_TYPE canBitRate);


// Change to the specified CAN peripheral mode.
// WARNING: BLOCKING CALL. EVENTUALLY WILL EXIT AFTER TIMEOUT,
// BUT THAT'S NOT YET IMPLEMENTED.

void RequestCanMode(enum CAN_MODES mode);



// Set the CAN mask for RB0 or RB1 with the specified mask value

void CanSetMask (enum CAN_MASK canMaskNum, CanHeader_t mask);



// Set the CAN filter for F0 - F5 (RB0 or RB1) with the specified filter value

void CanSetFilter (enum CAN_FILTER canFilterNum, CanHeader_t filter);

void CanTx2Isr (void);
void CanTx1Isr (void);
void CanTx0Isr (void);

void CanRx1Isr (void);
void CanRx0Isr (void);

void SendCanMessage (CanMessage_t* sendMessage);
void GetCanMessage (CanMessage_t* receiveMessage);


/*********************************************************************
 * Function:        bit CANSendMessage(unsigned long id,
 *                                  uint8_t *Data,
 *                                  uint8_t DataLen,
 *                                  enum CAN_TX_MSG_FLAGS MsgFlags)
 *
 * PreCondition:    None
 *
 * Input:           id          - CAN message identifier.
 *                                Only 11 or 29 bits may be used
 *                                depending on standard or extended
 *                                message type.
 *                  Data        - Data bytes of upto 8 bytes in length
 *                  DataLen     - Data length from 1 thru 8.
 *                  MsgFlags    - One or CAN_TX_MSG_FLAGS values ANDed
 *                                together
 *
 * Output:          If at least one empty transmit buffer is found,
 *                  given message is queued to be transmitted. If none
 *                  found FALSE value is returned.
 *
 * Side Effects:    None
 *
 ********************************************************************/

//bit CANSendMessage(unsigned long id,
//                     uint8_t *Data,
//                     uint8_t DataLen,
//                     enum CAN_TX_MSG_FLAGS MsgFlags);

/*********************************************************************
 * Function:        bit CANReceiveMessage(unsigned long *id,
 *                                  uint8_t *Data,
 *                                  uint8_t *DataLen,
 *                                  enum CAN_RX_MSG_FLAGS *MsgFlags)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          id          - CAN message identifier.
 *                  Data        - Data bytes of upto 8 bytes in length
 *                  DataLen     - Data length from 1 thru 8.
 *                  MsgFlags    - One or CAN_RX_MSG_FLAGS values ANDed
 *                                together
 *
 * Output:          If at least one full receive buffer is found,
 *                  it is extrated and returned. If none found FALSE
 *                  value is returned.
 *
 * Side Effects:    None
 *
 ********************************************************************/

//bit CANReceiveMessage(unsigned long* id,
//                        uint8_t *Data,
//                        uint8_t *DataLen,
//                        enum CAN_RX_MSG_FLAGS *MsgFlags);


//void	CanRxIsr		(void);

//void	CanErrorIsr		(void);





/*********************************************************************
 * #define'd Function Prototypes
 ********************************************************************/

// Set the operational mode, but do not wait for it to be implemented.
#define RequestCanModeNoWait(mode) (CANCON = mode)


// Get the operational mode.
#define CanGetMode() (CANCON & CAN_OP_MODE_BITS)


// Get the count of transmit errors (uint8_t)
#define CANGetTxErrorCount()    (TXERRCNT)


// Get the count of receive errors (uint8_t)
#define CanRxErrorCount()    (RXERRCNT)


// Check if the CAN peripheral mode is "Bus Off" due to errors
#define CanIsBusOff()	(TXB0)


// Check if the CAN transmit state is Error Passive
#define CanTxErrorPassive()	(TXBP)


// Check if the CAN receive state is Error Passive
#define CanRxErrorPassive()	(RXBP)


// Abort all CAN messages being transmitted
#define CanAbortAllTx()		(ABAT = 1)


// Check if there's a received messsage
#define CanRxReady()		(RXFUL || RXFUL)


// Check if there's an empty transmit buffer
#define CanTxReady()		(TXB0REQ || TXB1REQ || TXB2REQ)

#define DisableCanTx2Ints()	TXB2IE = 0
#define	EnableCanTx2Ints()	TXB2IE = 1

#define DisableCanTx1Ints()	TXB1IE = 0
#define	EnableCanTx1Ints()	TXB1IE = 1

#define DisableCanTx0Ints()	TXB0IE = 0
#define	EnableCanTx0Ints()	TXB0IE = 1


#define	ForceCanTx2Interrupt()	TXB2IF = 1
#define	ClearCanTx2Interrupt()	TXB2IF = 0

#define	ForceCanTx1Interrupt()	TXB1IF = 1
#define	ClearCanTx1Interrupt()	TXB1IF = 0

#define	ForceCanTx0Interrupt()	TXB0IF = 1
#define	ClearCanTx0Interrupt()	TXB0IF = 0

#define	CanTx2Full 		TXB2REQ
#define	CanTx1Full 		TXB1REQ
#define	CanTx0Full 		TXB0REQ


#define	ClearCanRx0Interrupt()	RXB0IF = 0; RXB0FUL = 0
#define DisableCanRx0Ints()		RXB0IE = 0
#define	EnableCanRx0Ints()		RXB0IE = 1

#define	ClearCanRx1Interrupt()	RXB1IF = 0; RXB1FUL = 0
#define DisableCanRx1Ints()		RXB1IE = 0
#define	EnableCanRx1Ints()		RXB1IE = 1


/*********************************************************************
 * Externs
 ********************************************************************/

extern	bit	canMessageAvailable;

//------------------------------------------------------------------------

#endif      // _CAN_H

