/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				interrupts.c
|
| Description:		Interrupt Handlers.
|
| Global Routines:	InitializeInterrupts
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <pic18.h>    	// Include for PICC-18
#include "pic18fxx8.h"	// Register definitions for the PIC18F458
#include "project.h"	// Includes for the project (Globals + misc)
#include "main.h"		// Includes for the mainline
#include "ints.h"		// Includes for interrupts
#include "excepts.h"	// Includes for exception handling
#include "ports.h"		// Includes for the port peripherals
#include "timer2.h"		// Includes for the Timer2 peripheral
#include "serial.h"		// Includes for the serial peripheral (UART )
#include "can.h"		// Includes for the CAN peripheral


//------------------------------------------------------------------------
//  Local function prototypes
//------------------------------------------------------------------------


/*---------------------------------------------------------------------------\
| Function:		HighPriorityISR
|
| Purpose:		Handle all of the high priority interrupts
|
| Discussion:	Checks to see what interrupted us. Why all this weird #ifdef
|				stuff? Because the PIC can't due recursive calls, and we have
|				an error function ("ThrowWarning") with a mainline, high
|				priority, and low priority flavors. In order for the compiler
|				to not yell at us, we need to figure out which interrupts
|				are which priority and #ifdef the correct error functions.
|				Ugly, but it's the best we can do (for now).
|
|				Why check to see if the interrupt is enabled? Because
|				any interrupt gets us here, we need to check to see
|				which interrupt needs servicing. But how can we tell if
|				if the interrupt flag is set but the interrupt is disabled?
|				We have to manually check. Stupid, but true.
|
| Expects:		all interrupts to be disabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE HIGH PRIORITY INTERRUPT SERVICE ROUTINE
// USING THE "interrupt" KEYWORD.


void interrupt HighPriorityISR (void)
{
	static bit hpTest;

	hpTest = FALSE;

	#if	INTS_TIMER2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( Timer2InterruptEnabled == TRUE
		 && Timer2InterruptPending == TRUE )
	{
		Timer2Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_INT0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( Int0InterruptEnabled == TRUE
		 && Int0InterruptPending == TRUE )
	{
		Int0Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_INT1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( Int1InterruptEnabled == TRUE
		 && Int1InterruptPending == TRUE )
	{
		Int1Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_ADC_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( AdcInterruptEnabled == TRUE
		 && AdcInterruptPending == TRUE )
	{
		AdcIsr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_SERIAL_TX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( SerialTxInterruptEnabled == TRUE
		 && SerialTxInterruptPending == TRUE )
	{
		SerialTxIsr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_SERIAL_RX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( SerialRxInterruptEnabled == TRUE
		 && SerialRxInterruptPending == TRUE )
	{
		SerialRxIsr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_TX2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( CanTxb2InterruptEnabled == TRUE
		 && CanTxb2InterruptPending == TRUE )
	{
		CanTx2Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_TX1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( CanTxb1InterruptEnabled == TRUE
		 && CanTxb1InterruptPending == TRUE )
	{
		CanTx1Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_TX0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( CanTxb0InterruptEnabled == TRUE
		 && CanTxb0InterruptPending == TRUE )
	{
		CanTx0Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_RX0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( CanRxb0InterruptEnabled == TRUE
		 && CanRxb0InterruptPending == TRUE )
	{
		CanRx0Isr();
		hpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_RX1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	if ( CanRxb1InterruptEnabled == TRUE
		 && CanRxb1InterruptPending == TRUE )
	{
		CanRx1Isr();
		hpTest = TRUE;
	}
	#endif

	// Throw a warning if we don't know why were interrupted.

	if (hpTest == FALSE)
		ThrowWarning_HP(WARN_ISR_UNKNOWN_HPI);
}



/*---------------------------------------------------------------------------\
| Function:		LowPriorityISR
|
| Purpose:		Handle all of the low prioirty interrupts
|
| Discussion:	Checks to see what interrupted us.
|
| Expects:		Low prioirty interrupts disabled but high prioirty may still
|				be enabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE LOW PRIOIRTY INTERRUPT SERVICE ROUTINE
// USING THE "interrupt low-priority" KEYWORDS.

void interrupt low_priority LowPriorityISR (void)
{
	static bit lpTest;

	lpTest = FALSE;

	#if	INTS_TIMER2_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( Timer2InterruptEnabled == TRUE
		 && Timer2InterruptPending == TRUE )
	{
		Timer2Isr();
		lpTest = TRUE;
	}
	#endif

	// Note: INT0 *can't* be low priority

	#if	INTS_INT1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( Int1InterruptEnabled == TRUE
		 && Int1InterruptPending == TRUE )
	{
		Int1Isr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_ADC_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( AdcInterruptEnabled == TRUE
		 && AdcInterruptPending == TRUE )
	{
		AdcIsr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_SERIAL_TX_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( SerialTxInterruptEnabled == TRUE
		 && SerialTxInterruptPending == TRUE )
	{
		SerialTxIsr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_SERIAL_RX_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( SerialRxInterruptEnabled == TRUE
		 && SerialRxInterruptPending == TRUE )
	{
		SerialRxIsr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_TX2_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( CanTxb2InterruptEnabled == TRUE
		 && CanTxb2InterruptPending == TRUE )
	{
		CanTx2Isr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_TX1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( CanTxb1InterruptEnabled == TRUE
		 && CanTxb1InterruptPending == TRUE )
	{
		CanTx1Isr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_TX0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( CanTxb0InterruptEnabled == TRUE
		 && CanTxb0InterruptPending == TRUE )
	{
		CanTx0Isr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_RX0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( CanRxb0InterruptEnabled == TRUE
		 && CanRxb0InterruptPending == TRUE )
	{
		CanRx0Isr();
		lpTest = TRUE;
	}
	#endif

	#if	INTS_CAN_RX1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	if ( CanRxb1InterruptEnabled == TRUE
		 && CanRxb1InterruptPending == TRUE )
	{
		CanRx1Isr();
		lpTest = TRUE;
	}
	#endif

	// Throw a warning if we don't know why were interrupted.

	if (lpTest == FALSE)
		ThrowWarning_LP(WARN_ISR_UNKNOWN_LPI);
}





/*---------------------------------------------------------------------------\
| Function:		InitializeInterrupts
|
| Purpose:		Initialize the PIC's interrupts.
|
| Discussion:	For each interrupt source (timer 2, ADC, CAN, etc), we're
|				going to choose the interrupt priority (low or high) and
|				whether we're going to enable or disable the interrupts.
|				We have to use compiler macros to do all this because the
|				the compiler will complain at us in other areas and heck,
|				it produces tight code (not that we need it in the
|				initialization routine). Once we're done setting priorities
|				and enables, we turn the interrupts on and we're off!
|
| Expects:		--
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializeInterrupts (void)
{
	// Turn on "high and low" priorities, not old peripheral/global interrupts
	// This will always be the case for all the nodes.

	EnablePrioritizedInts();

	// Although we define the <NAME>INTPRIORITY to be 0 (off), 1 (low) or
	// 2 (high) we can only set the prioirty bits in the PIC to 1 or 0;
	// so, 0 (off) and 1 (low) are set to 0 (low priority) and 2 (high) is
	// set to 1 (high priority)

	// Set the interrupt priority and enable or disable Timer 2 interrupts

	#if INTS_TIMER2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	Timer2PrioritySetting = HIGH_PRIORITY_INT;
	Timer2InterruptEnabled = TRUE;
	#elif  Timer2PrioritySetting == INTS_LOW_PRIORITY_INTERRUPT;
	Timer2PrioritySetting = LOW_PRIORITY_INT;
	Timer2InterruptEnabled = TRUE;
	#else
	Timer2PrioritySetting = LOW_PRIORITY_INT;
	Timer2InterruptEnabled = FALSE;
	#endif

	// Enable or disable INT0 (on PORTB) interrupts

	#if INTS_INT0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	// No prioirity setting for INT0 interrupts
	Int0InterruptEnabled = TRUE;
	#elif  INTS_INT0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	#error "INT0 interrupts may NOT be low priority!"
	Int0InterruptEnabled = FALSE;
	#else
	// No prioirity setting for INT0 interrupts
	Int0InterruptEnabled = FALSE;
	#endif

	// Set the interrupt priority and enable or disable Int1 (on PORTB) interrupts

	#if INTS_INT1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	Int1PrioritySetting = HIGH_PRIORITY_INT;
	Int1InterruptEnabled = TRUE;
	#elif  INTS_INT1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	Int1PrioritySetting = LOW_PRIORITY_INT;
	Int1InterruptEnabled = FALSE;
	#else
	Int1PrioritySetting = LOW_PRIORITY_INT;
	Int1InterruptEnabled = FALSE;
	#endif

	// Set the interrupt priority and enable or disable the ADC interrupt

	#if INTS_ADC_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	AdcPrioritySetting = HIGH_PRIORITY_INT;
	AdcInterruptEnabled = TRUE;
	#elif INTS_ADC_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	AdcPrioritySetting = LOW_PRIORITY_INT;
	AdcInterruptEnabled = FALSE;
	#else
	AdcPrioritySetting = LOW_PRIORITY_INT;
	AdcInterruptEnabled = FALSE;
	#endif


	// Set the interrupt priority and enable or disable the UART interrupts


	#if INTS_SERIAL_TX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	SerialTxPrioritySetting = HIGH_PRIORITY_INT;
	SerialTxInterruptEnabled = TRUE;
	#elif INTS_SERIAL_TX_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	SerialTxPrioritySetting = LOW_PRIORITY_INT;
	SerialTxInterruptEnabled = FALSE;
	#else
	SerialTxPrioritySetting = LOW_PRIORITY_INT;
	SerialTxInterruptEnabled = FALSE;
	#endif

	#if INTS_SERIAL_RX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	SerialRxPrioritySetting = HIGH_PRIORITY_INT;
	SerialRxInterruptEnabled = TRUE;
	#elif INTS_SERIAL_RX_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	SerialRxPrioritySetting = LOW_PRIORITY_INT;
	SerialRxInterruptEnabled = FALSE;
	#else
	SerialRxPrioritySetting = LOW_PRIORITY_INT;
	SerialRxInterruptEnabled = FALSE;
	#endif

	// Set the interrupt priority and enable or disable the CAN interrupts

	#if INTS_CAN_TX2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	CanTxb2PrioritySetting = HIGH_PRIORITY_INT;
	CanTxb2InterruptEnabled = TRUE;
	#elif INTS_CAN_TX2_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	CanTxb2PrioritySetting = LOW_PRIORITY_INT;
	CanTxb2InterruptEnabled = TRUE;
	#else
	CanTxb2PrioritySetting = LOW_PRIORITY_INT;
	CanTxb2InterruptEnabled = FALSE;
	#endif

	#if INTS_CAN_TX1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	CanTxb1PrioritySetting = HIGH_PRIORITY_INT;
	CanTxb1InterruptEnabled = TRUE;
	#elif INTS_CAN_TX1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	CanTxb1PrioritySetting = LOW_PRIORITY_INT;
	CanTxb1InterruptEnabled = TRUE;
	#else
	CanTxb1PrioritySetting = LOW_PRIORITY_INT;
	CanTxb1InterruptEnabled = FALSE;
	#endif

	#if INTS_CAN_TX0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	CanTxb0PrioritySetting = HIGH_PRIORITY_INT;
	CanTxb0InterruptEnabled = TRUE;
	#elif INTS_CAN_TX0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	CanTxb0PrioritySetting = LOW_PRIORITY_INT;
	CanTxb0InterruptEnabled = TRUE;
	#else
	CanTxb0PrioritySetting = LOW_PRIORITY_INT;
	CanTxb0InterruptEnabled = FALSE;
	#endif

	#if INTS_CAN_RX0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	CanRxb0PrioritySetting = HIGH_PRIORITY_INT;
	CanRxb0InterruptEnabled = TRUE;
	#elif INTS_CAN_RX0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	CanRxb0PrioritySetting = LOW_PRIORITY_INT;
	CanRxb0InterruptEnabled = TRUE;
	#else
	CanRxb0PrioritySetting = LOW_PRIORITY_INT;
	CanRxb0InterruptEnabled = FALSE;
	#endif

	#if INTS_CAN_RX1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	CanRxb1PrioritySetting = HIGH_PRIORITY_INT;
	CanRxb1InterruptEnabled = TRUE;
	#elif INTS_CAN_RX1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	CanRxb1PrioritySetting = LOW_PRIORITY_INT;
	CanRxb1InterruptEnabled = TRUE;
	#else
	CanRxb1PrioritySetting = LOW_PRIORITY_INT;
	CanRxb1InterruptEnabled = FALSE;
	#endif

	#if INTS_CAN_ERR_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
	CanErrPrioritySetting = HIGH_PRIORITY_INT;
	CanErrInterruptEnabled = TRUE;
	#elif INTS_CAN_ERR_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
	CanTxb2PrioritySetting = LOW_PRIORITY_INT;
	CanErrInterruptEnabled = TRUE;
	#else
	CanErrPrioritySetting = LOW_PRIORITY_INT;
	CanErrInterruptEnabled = FALSE;
	#endif

	// *** Turn on ints and away we go! ***

	EnableHighPriorityInts();
	EnableLowPriorityInts();
}

